#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <signal.h>
#include <fcntl.h>
#include <errno.h>
#include <sys/mman.h>
#include <sys/types.h> 

#include "fpga_awg.h"
#include "version.h"
#include "config.h"



//------------------------------------------------------------------

const uint32_t c_addrHousekeeping=0x40000000 ;

typedef struct {
	uint32_t id ;
	uint32_t DNApart1 ;
	uint32_t DNApart2 ;
        uint32_t dummy1 ;
	uint32_t dirP ;
	uint32_t dirN ;
	uint32_t outP ;
	uint32_t outN ;
	uint32_t inpP ;
	uint32_t inpN ;
        uint32_t dummy2 ;
        uint32_t dummy3 ;
	uint32_t LEDcntrl ;
} housekeepingReg_t;


#define FATAL do { fprintf(stderr, "Error at line %d, file %s (%d) [%s]\n", \
  __LINE__, __FILE__, errno, strerror(errno)); exit(1); } while(0)
 
#define MAP_SIZE 4096UL
#define MAP_MASK (MAP_SIZE - 1)

void checkFpgaId(uint32_t wantedID){
  void* map_base = (void*)(-1);
  int fd = -1;
  if((fd = open("/dev/mem", O_RDWR | O_SYNC)) == -1) FATAL;
  uint32_t addr = c_addrHousekeeping ;
  housekeepingReg_t* dataPtr=NULL;
  // Map one page
  map_base = mmap(0, MAP_SIZE, PROT_READ | PROT_WRITE, MAP_SHARED, fd, addr & ~MAP_MASK);
  if(map_base == (void *) -1) FATAL;
  dataPtr = map_base + (addr & MAP_MASK);
  uint32_t fpgaID = dataPtr->id ;
  if(fpgaID != wantedID ){
    printf("Warning: wrong FpgaID=%08X    should be %08X\n",fpgaID,wantedID) ;
    exit(1) ;
    }
  printf("right FpgaID=%08X    should be %08X\n",fpgaID,wantedID) ; 
  if (map_base != (void*)(-1)) {
    if(munmap(map_base, MAP_SIZE) == -1) FATAL;
    map_base = (void*)(-1);
    }
  }






//------------------------------------------------------------------

void printSIprefixed(double q, char *s){
  double q1=fabs(q) ;
 // printf("q1=%e ",q1) ;
  if( q1 >= 1e9   ) { fprintf(stderr,"%8.3f G%s",q/1e9  ,s) ; return ; }
  if( q1 >= 1e6   ) { fprintf(stderr,"%8.3f M%s",q/1e6  ,s) ; return ; }
  if( q1 >= 1e3   ) { fprintf(stderr,"%8.3f k%s",q/1e3  ,s) ; return ; }
  if( q1 >= 1e0   ) { fprintf(stderr,"%8.3f  %s",q/1e0  ,s) ; return ; }
  if( q1 >= 1e-3  ) { fprintf(stderr,"%8.3f m%s",q/1e-3 ,s) ; return ; }
  if( q1 >= 1e-6  ) { fprintf(stderr,"%8.3f u%s",q/1e-6 ,s) ; return ; }
  if( q1 >= 1e-9  ) { fprintf(stderr,"%8.3f n%s",q/1e-9 ,s) ; return ; }
  if( q1 >= 1e-12 ) { fprintf(stderr,"%8.3f p%s",q/1e-12,s) ; return ; }
                      fprintf(stderr,"%8.1e  %s",q/1e-12,s) ;
  }


void PMtest1(  ) ;

int main(int argc, char *argv[]){
  fprintf(stderr, "Hello FIR argc=%d\n",argc);
  checkFpgaId(0xAA00ABC6) ; // FIRsymSystolicUseArraysDSP48


  PMtest1() ;
  fprintf(stderr, "exit\n");   exit(1) ;
  }

   
