`default_nettype none

module red_pitaya_asg
(
   // DAC
   output  wire   [ 14-1: 0] dac_a_o         ,  //!< DAC data CHA
   output  wire   [ 14-1: 0] dac_b_o         ,  //!< DAC data CHB
   input   wire              dac_clk_i       ,  //!< DAC clock
   input wire                 dac_rstn_i      ,  //!< DAC reset - active low
   input wire                 trig_a_i        ,  //!< starting trigger CHA
   input wire                 trig_b_i        ,  //!< starting trigger CHB
   output wire                trig_out_o      ,  //!< notification trigger
//@@ossi insert ADC
  // ADC
   input wire      [ 14-1: 0] adc_a_i         ,  //!< ADC data CHA
   input wire      [ 14-1: 0] adc_b_i         ,  //!< ADC data CHB
   input wire                 adc_clk_i       ,  //!< ADC clock
   input wire                 adc_rstn_i      ,  //!< ADC reset - active low
   
   // System bus
   input wire                 sys_clk_i       ,  //!< bus clock
   input wire                 sys_rstn_i      ,  //!< bus reset - active low
   input wire      [ 32-1: 0] sys_addr_i      ,  //!< bus address
   input wire      [ 32-1: 0] sys_wdata_i     ,  //!< bus write data
   input wire      [  4-1: 0] sys_sel_i       ,  //!< bus write byte select
   input wire                 sys_wen_i       ,  //!< bus write enable
   input wire                 sys_ren_i       ,  //!< bus read enable
   output wire     [ 32-1: 0] sys_rdata_o     ,  //!< bus read data
   output wire                sys_err_o       ,  //!< bus error indicator
   output wire                sys_ack_o          //!< bus acknowledge signal
//@@ossi connection for trigger from modulator   
   ,
   output  wire[8-1:0] gpio 
);

wire [ 32-1: 0] addr         ;
wire [ 32-1: 0] wdata        ;
wire            wen          ;
wire            ren          ;
reg  [ 32-1: 0] rdata        ;
reg             err          ;
reg             ack          ;

reg [ 32-1: 0] debug2         ;
reg [ 32-1: 0] debug1         ;
assign gpio= 0 ;


reg  [14-1: 0] dacRega ;
reg  [14-1: 0] dacRegb ;
assign  dac_a_o = dacRega ;
assign  dac_b_o = dacRegb ;


//reg  signed [14-1: 0] filterOut ;
wire signed [  14-1: 0] FIRout         ;  
always @(posedge adc_clk_i) begin
    if(index==16'hFFFF) begin
      dacRega <= filterIn1   ;
      end
     else begin
      dacRega <= 0  ;
     end 
  dacRegb <= FIRout ;
  end


//http://stackoverflow.com/questions/16369698/how-to-pass-array-structure-between-two-verilog-modules

// parameter nTaps = 82 ; // at least two mul blocks not with DSP48

parameter nTaps = 64 ;
//parameter nTaps = 10 ;  // shows DSP48 blocks only
parameter xkWidth = 32 ;


wire signed  [  xkWidth-1: 0] filterIn1         ;
assign filterIn1 = { { xkWidth-14 {adc_a_i[14-1] } }, adc_a_i[14-1:0] } ; // sign extension



reg signed [ 32-1: 0] bkPar[nTaps-1:0] ;
wire [nTaps*32-1:0] bkFlat ;
genvar i;
  generate for (i = 0; i < nTaps; i = i+1) begin:flattenPar
    assign bkFlat[32*i +: 32] = bkPar[i]; 
  end endgenerate

reg [ 8-1: 0] scale ;

red_pitaya_FIRfilter   #(.nTaps(nTaps),.xkWidth(xkWidth)) theFIRfilter
( .clk_i(dac_clk_i),
  .rstn_i(dac_rstn_i),
  .scale_i(scale),
  .FIRin_i(filterIn1),
  .bkFlat_i(bkFlat),
  .FIRout_o(FIRout)   );

//---------------------------------------------------------------------------------
//
//  System bus connection

reg  [3-1: 0] ren_dly ;
reg           ack_dly ;

reg  [16-1: 0] index ;

always @(posedge dac_clk_i) begin
   if (dac_rstn_i == 1'b0) begin
      ren_dly    <=  3'h0    ;
      ack_dly    <=  1'b0    ;
    end
   else begin
      if (wen) begin
// ossi         
         if (addr[19:0]==20'h30)  index <= wdata[16-1: 0] ;
         if (addr[19:0]==20'h34)  bkPar[index] <= wdata[32-1: 0] ;
         if (addr[19:0]==20'h38)  scale <= wdata[8-1: 0] ;
      end

      ren_dly <= {ren_dly[3-2:0], ren};
      ack_dly <=  ren_dly[3-1] || wen ;
   end
end

always @(*) begin
   err <= 1'b0 ;

   casez (addr[19:0])
 //@@ossi
      20'h00040 : begin ack <= 1'b1;          rdata <=  debug1    ; end
	 20'h00044 : begin ack <= 1'b1;          rdata <=  debug2    ; end
     20'h00048 : begin ack <= 1'b1;          rdata <=  32'h13547349  ; end
     20'h0004C : begin ack <= 1'b1;          rdata <=  index  ; end
     20'h00050 : begin ack <= 1'b1;          rdata <=  bkPar[index]  ; end
     20'h00054 : begin ack <= 1'b1;          rdata <=  nTaps  ; end
     default : begin ack <= 1'b1;          rdata <=  32'h0                             ; end
   endcase
end




// bridge between DAC and sys clock
bus_clk_bridge i_bridge
(
   .sys_clk_i     (  sys_clk_i      ),
   .sys_rstn_i    (  sys_rstn_i     ),
   .sys_addr_i    (  sys_addr_i     ),
   .sys_wdata_i   (  sys_wdata_i    ),
   .sys_sel_i     (  sys_sel_i      ),
   .sys_wen_i     (  sys_wen_i      ),
   .sys_ren_i     (  sys_ren_i      ),
   .sys_rdata_o   (  sys_rdata_o    ),
   .sys_err_o     (  sys_err_o      ),
   .sys_ack_o     (  sys_ack_o      ),

   .clk_i         (  dac_clk_i      ),
   .rstn_i        (  dac_rstn_i     ),
   .addr_o        (  addr           ),
   .wdata_o       (  wdata          ),
   .wen_o         (  wen            ),
   .ren_o         (  ren            ),
   .rdata_i       (  rdata          ),
   .err_i         (  err            ),
   .ack_i         (  ack            )
);

endmodule
`default_nettype wire

 
