`default_nettype none

module red_pitaya_FIRfilter
#( parameter nTaps = 16,parameter xkWidth=14 )
( input wire            clk_i       ,
  input wire            rstn_i      ,
  input wire[8-1:0]     scale_i ,
  input wire signed [ xkWidth-1:0] FIRin_i       , 
  input wire [nTaps*32-1:0] bkFlat_i ,
  output wire signed [ 14-1:0] FIRout_o
  );

reg signed [18-1:0] bk [nTaps-1:0];

integer i;
always @(posedge clk_i) begin
  for (i = 0; i <nTaps ; i = i+1)
    bk[i] <= bkFlat_i[32*i +: 32] ;
  end ;  
      

parameter Awidth = 30 ;
parameter Dwidth = 25 ;
parameter ADwidth = 25 ;


parameter Bwidth = 18 ;
parameter Cwidth = 32 ;
//parameter Mwidth = 72 ;
parameter Mwidth = 43 ;
parameter Pwidth = 48 ;

//--------------------------------------------------------------------

wire signed [ Awidth-1:0] FIRinputA ;
wire signed [ Dwidth-1:0] FIRinputD ; 
assign FIRinputA = FIRin_i ;
assign FIRinputD = FIRin_i ;

//--------------------------------------------------------------------

reg signed [Awidth-1: 0] A1[nTaps-1:0] ;
reg signed [Awidth-1: 0] A2[nTaps-1:0] ;
reg signed [Dwidth-1: 0] D1[nTaps-1:0] ;
reg signed [ADwidth-1: 0] AD1[nTaps-1:0] ;
reg signed [Mwidth-1: 0] M1[nTaps-1:0] ;
reg signed [Pwidth-1: 0] P1[nTaps-1:0] ;   

//assign FIRout_o = P1[0] ;

integer k ;
always @(posedge clk_i) begin
  for(k=0 ; k<nTaps-1 ; k=k+1 ) begin 
    A1[k] <= A2[k+1] ;
    A2[k] <= A1[k] ;
    D1[k] <= FIRinputD ;
    AD1[k] <= $signed(A2[k]+D1[k]) ;
    M1[k] <= AD1[k]*bk[k] ;
    P1[k] <= M1[k] + P1[k+1]  ;
    end
  end

always @(posedge clk_i) begin
  A1[nTaps-1] <= FIRinputA ;
  A2[nTaps-1] <= A1[nTaps-1] ;
  D1[nTaps-1] <= FIRinputD ;
  AD1[nTaps-1] <= $signed(A2[nTaps-1]+D1[nTaps-1]) ;
  M1[nTaps-1] <= AD1[nTaps-1]*bk[nTaps-1] ;
  P1[nTaps-1] <= M1[nTaps-1]   ; 
  end

outputSelector select1( 
  .clk_i(clk_i), 
  .ena_i(1), 
  .FIRsum_i(P1[0]),
  .selector_i(scale_i),
  .FIRout_o(FIRout_o) 
   ) ;
    
//---------------------------------------------------------------------------------

endmodule

`default_nettype wire

