`default_nettype none
//https://groups.google.com/forum/#!topic/comp.lang.verilog/RwF6aRdlP2I

module red_pitaya_asg
(
   // DAC
   output  wire    [ 14-1: 0] dac_a_o         ,  //!< DAC data CHA
   output  wire    [ 14-1: 0] dac_b_o         ,  //!< DAC data CHB
   input wire                 dac_clk_i       ,  //!< DAC clock
   input wire                 dac_rstn_i      ,  //!< DAC reset - active low
   input wire                 trig_a_i        ,  //!< starting trigger CHA
   input wire                 trig_b_i        ,  //!< starting trigger CHB
   output wire                trig_out_o      ,  //!< notification trigger
//@@ossi insert ADC
  // ADC
   input wire      [ 14-1: 0] adc_a_i         ,  //!< ADC data CHA
   input wire      [ 14-1: 0] adc_b_i         ,  //!< ADC data CHB
   input wire                 adc_clk_i       ,  //!< ADC clock
   input wire                 adc_rstn_i      ,  //!< ADC reset - active low
   
   // System bus
   input wire                 sys_clk_i       ,  //!< bus clock
   input wire                 sys_rstn_i      ,  //!< bus reset - active low
   input wire      [ 32-1: 0] sys_addr_i      ,  //!< bus address
   input wire      [ 32-1: 0] sys_wdata_i     ,  //!< bus write data
   input wire      [  4-1: 0] sys_sel_i       ,  //!< bus write byte select
   input wire                 sys_wen_i       ,  //!< bus write enable
   input wire                 sys_ren_i       ,  //!< bus read enable
   output wire     [ 32-1: 0] sys_rdata_o     ,  //!< bus read data
   output wire                sys_err_o       ,  //!< bus error indicator
   output wire                sys_ack_o          //!< bus acknowledge signal
//@@ossi connection for trigger from modulator   
   ,
   output wire [8-1:0] gpio 
);



wire [ 32-1: 0] addr         ;
wire [ 32-1: 0] wdata        ;
wire            wen          ;
wire            ren          ;
reg  [ 32-1: 0] rdata        ;
reg             err          ;
reg             ack          ;

reg [ 32-1: 0] debug2         ;
reg [ 32-1: 0] debug1         ;
assign gpio= 0 ;

wire  [14-1: 0] dac_a_dummy ;
wire  [14-1: 0] dac_b_dummy ;
wire  [14-1: 0] dacRega ;
reg  [14-1: 0] dacRegb ;

assign  dac_a_o = dacRega ;
assign  dac_b_o = dacRegb ;

/*
parameter FIRsize   = 8192 ;
parameter adrWidth = 13 ;
parameter xkWidth  = 14 ;
*/

parameter FIRsize   = 16384 ;
parameter adrWidth = 14 ;
parameter xkWidth  = 14 ;
 


reg  [16-1: 0] index ;
reg  [8-1: 0] scale ;
reg  [8-1: 0] FIRoptions ;

reg                    buf_a_we         ;
reg   [ adrWidth-1: 0] buf_a_addr       ;
wire strobe ;


reg signed  [ xkWidth-1: 0] testSignal  ;
reg signed  [ 16-1: 0] testCounter  ;
reg signed  [ 16-1: 0] testSignalPeriod  ;

always @(posedge dac_clk_i) begin
  if (strobe ) begin
    dacRegb <= testSignal ;
    if(testCounter==testSignalPeriod) begin
      testCounter <= 0 ;
      testSignal <= 8000 ;
      end
     else begin
      testCounter <= testCounter + 1 ;
      testSignal <= 0 ;
      end
    end  
  end

reg signed [ xkWidth-1: 0] input_A_  ;

always @(posedge dac_clk_i) begin
  if(index== 16'hFFFF) begin 
    input_A_  <= testSignal ;
    end
   else begin
    input_A_  <=$signed( adc_a_i) ;
    end
  end

reg  [16-1: 0] FIRlength ;

red_pitaya_FIRfilter 
 #(.size( FIRsize ), .adrWidth(adrWidth), .xkWidth(xkWidth) ) 
 filter1
( .clk_i(dac_clk_i)       ,  
  .rstn_i(dac_rstn_i)     ,
  .buf_we_i(buf_a_we)        ,  //!< buffer write enable
  .buf_addr_i(buf_a_addr)      ,  //!< buffer address
  .buf_wdata_i(wdata[32-1:0] ) ,  //!< buffer write data
  .scale_i(scale),
  .length_i(FIRlength), 
  .FIRoptions_i(FIRoptions), 
  .filterIn_i(input_A_), 
  .filterOut_o(dacRega ),
  .sample_o(strobe) 
  );
 
//---------------------------------------------------------------------------------

wire [31:0] TimeAndDate ;
USR_ACCESSE2 mUsrAccess ( .DATA ( TimeAndDate ), .CFGCLK (), .DATAVALID () );
// in XDC file: set_property BITSTREAM.CONFIG.USR_ACCESS TIMESTAMP [current_design]  

//
//  System bus connection

reg  [3-1: 0] ren_dly ;
reg           ack_dly ;

localparam RSZ = 14 ;  // RAM size 2^RSZ

always @(posedge dac_clk_i) begin
   buf_a_we   <= wen && (addr[19:RSZ+2] == 'h1);
   buf_a_addr <= addr[adrWidth+1:2] ;  // address timing violation
   end


always @(posedge dac_clk_i) begin
   if (dac_rstn_i == 1'b0) begin
      ren_dly    <=  3'h0    ;
      ack_dly    <=  1'b0    ;
    end
   else begin
      if (wen) begin 
        if (addr[19:0]==20'h40)  FIRoptions <= wdata[8-1: 0] ;
        if (addr[19:0]==20'h44)  testSignalPeriod <= wdata[16-1: 0] ;
        if (addr[19:0]==20'h48)  scale <= wdata[8-1: 0] ;
        if (addr[19:0]==20'h4C)  index <= wdata[16-1: 0] ;
        if (addr[19:0]==20'h50)  FIRlength <= wdata[16-1: 0] ;
        end
      ren_dly <= {ren_dly[3-2:0], ren};
      ack_dly <=  ren_dly[3-1] || wen ;
   end
end

always @(*) begin
   err <= 1'b0 ;

   casez (addr[19:0])
     20'h00040 : begin ack <= 1'b1;          rdata <=  TimeAndDate  ; end
     20'h00054 : begin ack <= 1'b1;          rdata <=  FIRsize  ; end
     default : begin ack <= 1'b1;          rdata <=  32'h0      ; end
   endcase
end




// bridge between DAC and sys clock
bus_clk_bridge i_bridge
(
   .sys_clk_i     (  sys_clk_i      ),
   .sys_rstn_i    (  sys_rstn_i     ),
   .sys_addr_i    (  sys_addr_i     ),
   .sys_wdata_i   (  sys_wdata_i    ),
   .sys_sel_i     (  sys_sel_i      ),
   .sys_wen_i     (  sys_wen_i      ),
   .sys_ren_i     (  sys_ren_i      ),
   .sys_rdata_o   (  sys_rdata_o    ),
   .sys_err_o     (  sys_err_o      ),
   .sys_ack_o     (  sys_ack_o      ),

   .clk_i         (  dac_clk_i      ),
   .rstn_i        (  dac_rstn_i     ),
   .addr_o        (  addr           ),
   .wdata_o       (  wdata          ),
   .wen_o         (  wen            ),
   .ren_o         (  ren            ),
   .rdata_i       (  rdata          ),
   .err_i         (  err            ),
   .ack_i         (  ack            )
);

endmodule

`default_nettype wire