module FirDsp48TypA(
   input                        CLK , 
   input wire signed  [ 30-1:0] A_i , 
   input wire signed  [ 18-1:0] B_i ,
   output wire signed [ 48-1:0] P_o ,
   output wire signed  [ 48-1:0] PCOUT_o ,
   output wire signed  [ 30-1:0] ACOUT_o  
   );

// macro template from: virtex6_hdl.pdf

wire Const0 ;   assign Const0=0 ;
wire Const1 ;   assign Const1=1 ;  

//---------------------------------------------------------------------------------
wire [ 7-1:0] OPMODE ; assign OPMODE=7'b0000101 ; // P=A*B+0
wire [ 5-1:0] INMODE ; assign INMODE=5'b00000 ;
wire [ 4-1:0] ALUMODE ;  assign ALUMODE=0 ;
wire [ 3-1:0] CARRYINSEL ; assign CARRYINSEL=0 ;



// DSP48E1: 48-bit Multi-Functional Arithmetic Block
// Virtex-6
// Xilinx HDL Libraries Guide, version 12.4
DSP48E1 #(
// Feature Control Attributes: Data Path Selection
.A_INPUT("DIRECT"), // Selects A input source, "DIRECT" (A port) or "CASCADE" (ACIN port)
.B_INPUT("DIRECT"), // Selects B input source, "DIRECT" (B port) or "CASCADE" (BCIN port)
.USE_DPORT("FALSE"), // Select D port usage (TRUE or FALSE)
.USE_MULT("MULTIPLY"), // Select multiplier usage ("MULTIPLY", "DYNAMIC", or "NONE")
// Pattern Detector Attributes: Pattern Detection Configuration
.AUTORESET_PATDET("NO_RESET"), // "NO_RESET", "RESET_MATCH", "RESET_NOT_MATCH"
.MASK(48'h3fffffffffff), // 48-bit mask value for pattern detect (1=ignore)
.PATTERN(48'h000000000000), // 48-bit pattern match for pattern detect
.SEL_MASK("MASK"), // "C", "MASK", "ROUNDING_MODE1", "ROUNDING_MODE2"
.SEL_PATTERN("PATTERN"), // Select pattern value ("PATTERN" or "C")
.USE_PATTERN_DETECT("NO_PATDET"), // Enable pattern detect ("PATDET" or "NO_PATDET")
// Register Control Attributes: Pipeline Register Configuration
.ACASCREG(2), // Number of pipeline stages between A/ACIN and ACOUT (0, 1 or 2)
.ADREG(0), // Number of pipeline stages for pre-adder (0 or 1)
.ALUMODEREG(0), // Number of pipeline stages for ALUMODE (0 or 1)
.AREG(2), // Number of pipeline stages for A (0, 1 or 2)
.BCASCREG(0), // Number of pipeline stages between B/BCIN and BCOUT (0, 1 or 2)
.BREG(0), // Number of pipeline stages for B (0, 1 or 2)
.CARRYINREG(0), // Number of pipeline stages for CARRYIN (0 or 1)
.CARRYINSELREG(0), // Number of pipeline stages for CARRYINSEL (0 or 1)
.CREG(0), // Number of pipeline stages for C (0 or 1)
.DREG(0), // Number of pipeline stages for D (0 or 1)
.INMODEREG(0), // Number of pipeline stages for INMODE (0 or 1)
.MREG(1), // Number of multiplier pipeline stages (0 or 1)
.OPMODEREG(0), // Number of pipeline stages for OPMODE (0 or 1)
.PREG(1), // Number of pipeline stages for P (0 or 1)
.USE_SIMD("ONE48") // SIMD selection ("ONE48", "TWO24", "FOUR12")
)


DSP48E1_inst (
// Cascade: 30-bit (each) output Cascade Ports
.ACOUT(ACOUT_o), // 30-bit output A port cascade output
.BCOUT(BCOUT), // 18-bit output B port cascade output
.CARRYCASCOUT(CARRYCASCOUT), // 1-bit output Cascade carry output
.MULTSIGNOUT(MULTSIGNOUT), // 1-bit output Multiplier sign cascade output
.PCOUT(PCOUT_o), // 48-bit output Cascade output
// Control: 1-bit (each) output Control Inputs/Status Bits
.OVERFLOW(OVERFLOW), // 1-bit output Overflow in add/acc output
.PATTERNBDETECT(PATTERNBDETECT), // 1-bit output Pattern bar detect output
.PATTERNDETECT(PATTERNDETECT), // 1-bit output Pattern detect output
.UNDERFLOW(UNDERFLOW), // 1-bit output Underflow in add/acc output
// Data: 4-bit (each) output Data Ports
.CARRYOUT(CARRYOUT), // 4-bit output Carry output
.P(P_o), // 48-bit output Primary data output
// Cascade: 30-bit (each) input Cascade Ports
.ACIN(Const0), // 30-bit input A cascade data input
.BCIN(Const0), // 18-bit input B cascade input
.CARRYCASCIN(Const0), // 1-bit input Cascade carry input
.MULTSIGNIN(Const0), // 1-bit input Multiplier sign input
.PCIN(Const0), // 48-bit input P cascade input
// Control: 4-bit (each) input Control Inputs/Status Bits
.ALUMODE(ALUMODE), // 4-bit input ALU control input
.CARRYINSEL(CARRYINSEL), // 3-bit input Carry select input
.CEINMODE(Const0), // 1-bit input Clock enable input for INMODEREG
.CLK(CLK), // 1-bit input Clock input
.INMODE(INMODE), // 5-bit input INMODE control input
.OPMODE(OPMODE), // 7-bit input Operation mode input
.RSTINMODE(Const0), // 1-bit input Reset input for INMODEREG
// Data: 30-bit (each) input Data Ports
.A(A_i), // 30-bit input A data input
.B(B_i), // 18-bit input B data input
.C(Const0), // 48-bit input C data input
.CARRYIN(Const0), // 1-bit input Carry input signal
.D(Const0), // 25-bit input D data input
// Reset/Clock Enable: 1-bit (each) input Reset/Clock Enable Inputs
.CEA1(Const1), // 1-bit input Clock enable input for 1st stage AREG
.CEA2(Const1), // 1-bit input Clock enable input for 2nd stage AREG
.CEAD(Const0), // 1-bit input Clock enable input for ADREG
.CEALUMODE(Const0), // 1-bit input Clock enable input for ALUMODERE
.CEB1(Const0), // 1-bit input Clock enable input for 1st stage BREG
.CEB2(Const0), // 1-bit input Clock enable input for 2nd stage BREG
.CEC(Const0), // 1-bit input Clock enable input for CREG
.CECARRYIN(Const0), // 1-bit input Clock enable input for CARRYINREG
.CECTRL(Const0), // 1-bit input Clock enable input for OPMODEREG and CARRYINSELREG
.CED(Const0), // 1-bit input Clock enable input for DREG
.CEM(Const1), // 1-bit input Clock enable input for MREG
.CEP(Const1), // 1-bit input Clock enable input for PREG
.RSTA(Const0), // 1-bit input Reset input for AREG
.RSTALLCARRYIN(Const0), // 1-bit input Reset input for CARRYINREG
.RSTALUMODE(Const0), // 1-bit input Reset input for ALUMODEREG
.RSTB(Const0), // 1-bit input Reset input for BREG
.RSTC(Const0), // 1-bit input Reset input for CREG
.RSTCTRL(Const0), // 1-bit input Reset input for OPMODEREG and CARRYINSELREG
.RSTD(Const0), // 1-bit input Reset input for DREG and ADREG
.RSTM(Const0), // 1-bit input Reset input for MREG
.RSTP(Const0) // 1-bit input Reset input for PREG
);
// End of DSP48E1_inst instantiation

endmodule
